package com.prlab.idpserver.hkdf;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

public class HMAC {

    private byte[] key;
    private byte[] msg;
    private byte[] opad = new byte[64];
    private byte[] ipad = new byte[64];

    public HMAC(byte[] key) throws NoSuchAlgorithmException {
        setKey(key);
        for (int i = 0; i < opad.length; ++i) {
            opad[i] = 0x5c;
            ipad[i] = 0x36;
        }
    }
    public void setKey(byte[] key) throws NoSuchAlgorithmException {
        if(key == null){
            return;
        }
        else if(key.length < 64){
            this.key = padding(key, 64);
        }
        else{
            this.key = hash(key);
        }
    }
    public void update(byte[] m ){
        if(this.msg == null){
            this.msg = m;
        }
        else{
            this.msg = concat(this.msg, m);
        }
    }
    // force use SHA256
    public byte[] doFinal() throws NoSuchAlgorithmException{
        byte[] output = hash(concat(xor(this.key, opad), hash((concat(xor(this.key, ipad), this.msg)))));
        this.msg = null;
        return output;
    }
    private byte[] padding(byte[] key, int length){
        byte[] ret = new byte[length];
        for(int i = 0; i < length; i++){
            if(i >= key.length){
                ret[i] = 0;
            }
            else{
                ret[i] = key[i];
            }
        }
        return ret;
    }
    public static byte[] xor(byte[] a, byte[] b) {
        byte[] result = new byte[Math.max(a.length, b.length)];
        for (int i = 0; i < result.length; ++i) {
            result[i] = (byte) (0xff & ((int) a[i] ^ (int) b[i]));
        }
        return result;
    }
    public static byte[] concat(byte[] a, byte[] b){

        byte[] ret = new byte[a.length + b.length];
        for (int i = 0; i < a.length; i++){
            ret[i] = a[i];         
        }
        for(int i = 0; i < b.length; i ++){
            ret[ i + a.length ] = b[i];
        }
        return ret; 
    }
    private byte[] hash(byte[] m) throws NoSuchAlgorithmException{
         MessageDigest md = MessageDigest.getInstance("SHA-256");
         return md.digest(m);
    }
}
